@@ -213,7 +213,7 @@ t_cached_object *search_cache(t_session *session, char *file) {
 							}
 							copy_ip(&(object->last_ip), &(session->ip_address));
 						}
-						(object->in_use)++;
+						object->in_use++;
 						result = object;
 					} else if (object->in_use <= 0) {
 						remove_from_cache(object, i);
@@ -236,7 +236,7 @@ void done_with_cached_object(t_cached_object *object, bool remove_object) {
 	if (remove_object) {
 		object->deadline = 0;
 	}
-	(object->in_use)--;
+	object->in_use--;
 }
 
 void check_cache(time_t now) {
