@@ -32,7 +32,7 @@
 #include "envir.h"
 #include "log.h"
 
-#define MAX_CGI_DELAY_TIMER MINUTE
+#define MAX_CGI_DELAY_TIMER (int)MINUTE / TASK_RUNNER_INTERVAL
 #define DUMMY_BUFFER_SIZE 64 * KILOBYTE
 
 static int delay_timer = 0;
@@ -109,6 +109,8 @@ t_connect_to *select_connect_to(t_fcgi_server *fcgi_server, t_ip_addr *client_ip
 	t_cgi_session *cgi_session;
 	bool search_new_fcgi_server = true;
 	unsigned char i;
+	time_t now;
+
 
 	if ((fcgi_server == NULL) || (client_ip == NULL)) {
 		return NULL;
@@ -120,6 +122,8 @@ t_connect_to *select_connect_to(t_fcgi_server *fcgi_server, t_ip_addr *client_ip
 		return fcgi_server->connect_to;
 	}
 
+	now = time(NULL);
+
 	i = index_by_ip(client_ip);
 	pthread_mutex_lock(&fcgi_server->cgi_session_mutex[i]);
 
@@ -128,9 +132,9 @@ t_connect_to *select_connect_to(t_fcgi_server *fcgi_server, t_ip_addr *client_ip
 	cgi_session = fcgi_server->cgi_session_list[i];
 	while (cgi_session != NULL) {
 		if (same_ip(&(cgi_session->client_ip), client_ip)) {
-			if (cgi_session->timer - delay_timer > 0) {
+			if (now < cgi_session->session_timeout) {
 				if (cgi_session->connect_to->available) {
-					cgi_session->timer = fcgi_server->session_timeout + delay_timer;
+					cgi_session->session_timeout = now + fcgi_server->session_timeout;
 					connect_to = cgi_session->connect_to;
 				} else {
 					/* Make sure it won't match the next round */
@@ -158,7 +162,7 @@ t_connect_to *select_connect_to(t_fcgi_server *fcgi_server, t_ip_addr *client_ip
 			if ((cgi_session = (t_cgi_session*)malloc(sizeof(t_cgi_session))) != NULL) {
 				copy_ip(&(cgi_session->client_ip), client_ip);
 				cgi_session->connect_to = connect_to;
-				cgi_session->timer = fcgi_server->session_timeout + delay_timer;
+				cgi_session->session_timeout = now + fcgi_server->session_timeout;
 
 				cgi_session->next = fcgi_server->cgi_session_list[i];
 				fcgi_server->cgi_session_list[i] = cgi_session;
@@ -171,61 +175,63 @@ t_connect_to *select_connect_to(t_fcgi_server *fcgi_server, t_ip_addr *client_ip
 	return connect_to;
 }
 
-void check_load_balancer(t_config *config) {
+void check_load_balancer(t_config *config, time_t now) {
 	t_fcgi_server *fcgi_server;
 	t_cgi_session *cgi_session, *last, *next = NULL;
 	t_connect_to  *connect_to = NULL;
 	int i, sock;
 
-	if (++delay_timer == MAX_CGI_DELAY_TIMER) {
-		fcgi_server = config->fcgi_server;
-		while (fcgi_server != NULL) {
-			/* Check session timeouts
-			 */
-			for (i = 0; i < 256; i++) {
-				pthread_mutex_lock(&fcgi_server->cgi_session_mutex[i]);
+	if (++delay_timer < MAX_CGI_DELAY_TIMER) {
+		return;
+	}
 
-				last = NULL;
-				cgi_session = fcgi_server->cgi_session_list[i];
-				while (cgi_session != NULL) {
-					next = cgi_session->next;
+	fcgi_server = config->fcgi_server;
+	while (fcgi_server != NULL) {
+		/* Check session timeouts
+		 */
+		for (i = 0; i < 256; i++) {
+			pthread_mutex_lock(&fcgi_server->cgi_session_mutex[i]);
 
-					if (((cgi_session->timer -= MAX_CGI_DELAY_TIMER) <= 0) || (cgi_session->connect_to->available == false)) {
-						if (last == NULL) {
-							fcgi_server->cgi_session_list[i] = next;
-						} else {
-							last->next = next;
-						}
-						free(cgi_session);
+			last = NULL;
+			cgi_session = fcgi_server->cgi_session_list[i];
+			while (cgi_session != NULL) {
+				next = cgi_session->next;
+
+				if ((now > cgi_session->session_timeout) || (cgi_session->connect_to->available == false)) {
+					if (last == NULL) {
+						fcgi_server->cgi_session_list[i] = next;
 					} else {
-						last = cgi_session;
+						last->next = next;
 					}
-					cgi_session = next;
+					free(cgi_session);
+				} else {
+					last = cgi_session;
 				}
-
-				pthread_mutex_unlock(&fcgi_server->cgi_session_mutex[i]);
+				cgi_session = next;
 			}
 
-			/* Check if offline FastCGI servers are available again
-			 */
-			connect_to = fcgi_server->connect_to;
-			do { 
-				if (connect_to->available == false) {
-					if ((sock = connect_to_fcgi_server(connect_to)) != -1) {
-						close(sock);
-						connect_to->available = true;
-					} else {
-						log_string(config->system_logfile, "FastCGI server %s is still (partially) unavailable", fcgi_server->fcgi_id);
-					}
-				}
-				connect_to = connect_to->next;
-			} while (connect_to != fcgi_server->connect_to);
-
-			fcgi_server = fcgi_server->next;
+			pthread_mutex_unlock(&fcgi_server->cgi_session_mutex[i]);
 		}
 
-		delay_timer = 0;
+		/* Check if offline FastCGI servers are available again
+		 */
+		connect_to = fcgi_server->connect_to;
+		do { 
+			if (connect_to->available == false) {
+				if ((sock = connect_to_fcgi_server(connect_to)) != -1) {
+					close(sock);
+					connect_to->available = true;
+				} else {
+					log_string(config->system_logfile, "FastCGI server %s is still (partially) unavailable", fcgi_server->fcgi_id);
+				}
+			}
+			connect_to = connect_to->next;
+		} while (connect_to != fcgi_server->connect_to);
+
+		fcgi_server = fcgi_server->next;
 	}
+
+	delay_timer = 0;
 }
 
 /*
