@@ -619,6 +619,9 @@ void print_client_list(FILE *fp) {
 				if (inet_ntop(client->session->ip_address.family, &(client->session->ip_address.value), ip_address, MAX_IP_STR_LEN) != NULL) {
 					fprintf(fp, "  IP-address  : %s\n", ip_address);
 				}
+				if (client->session->last_host != NULL) {
+					fprintf(fp, "  Hostname    : %s\n", *(client->session->last_host->hostname.item));
+				}
 				fprintf(fp, "  Socket      : %d\n", client->session->client_socket);
 				if (client->session->remote_user != NULL) {
 					fprintf(fp, "  Remote user : %s\n", client->session->remote_user);
