@@ -886,6 +886,9 @@ int serve_client(t_session *session) {
 	if (session->hostname != NULL) {
 		if ((host_record = get_hostrecord(session->config->first_host, session->hostname, session->binding)) != NULL) {
 			session->host = host_record;
+#ifdef HAVE_COMMAND
+			session->last_host = host_record;
+#endif
 		}
 	}
 	session->host->access_time = session->time;
@@ -934,7 +937,11 @@ int serve_client(t_session *session) {
 #ifdef HAVE_TOOLKIT
 	/* URL toolkit
 	 */
+#ifdef HAVE_SSL
 	init_toolkit_options(&options, session->host->website_root, session->config->url_toolkit, session->binding->use_ssl);
+#else
+	init_toolkit_options(&options, session->host->website_root, session->config->url_toolkit, false);
+#endif
 
 	if ((session->request_method != PUT) && (session->request_method != DELETE)) {
 		for (i = 0; i < session->host->toolkit_rules.size; i++) {
