@@ -190,7 +190,7 @@ void remove_charlist(t_charlist *list) {
 	if (list != NULL) {
 		if (list->size > 0) {
 			do {
-				(list->size)--;
+				list->size--;
 				sfree(*(list->item + list->size));
 			} while (list->size > 0);
 			sfree(list->item);
