@@ -814,7 +814,7 @@ static bool system_setting(char *key, char *value, t_config *config) {
 			}
 		}
 	} else if (strcmp(key, "hideproxy") == 0) {
-		if (parse_iplist(value, &(config->hide_proxy)) == 0) {
+		if (parse_iplist(value, &(config->hide_proxy)) != -1) {
 			return true;
 		}
 	} else if (strcmp(key, "kickonban") == 0) {
