@@ -30,7 +30,6 @@
 
 #define MAX_START_FILE_LENGTH 32
 
-
 typedef enum { root, part } t_pathmatch;
 typedef enum { no_auth, basic, digest } t_auth_method;
 typedef enum { hiawatha, common, extended } t_log_format;
@@ -38,6 +37,23 @@ typedef enum { hiawatha, common, extended } t_log_format;
 typedef enum { windows, cygwin } t_platform;
 #endif
 
+#ifdef HAVE_MONITOR
+typedef struct type_monitor_srv_stats {
+	int simultaneous_connections;
+} t_monitor_srv_stats;
+
+typedef struct type_monitor_host_stats {
+	int requests;
+	off_t bytes_sent;
+	int bans;
+	int exploit_attempts;
+
+	int result_forbidden;
+	int result_not_found;
+	int result_internal_error;
+} t_monitor_host_stats;
+#endif
+
 typedef struct type_deny_body {
 	regex_t pattern;
 	struct type_deny_body *next;
@@ -61,7 +77,7 @@ typedef struct type_connect_to {
 typedef struct type_cgi_session {
 	t_ip_addr     client_ip;
 	t_connect_to  *connect_to;
-	int           timer;
+	time_t        session_timeout;
 
 	struct type_cgi_session *next;
 } t_cgi_session;
@@ -120,10 +136,9 @@ typedef struct type_directory {
 	char          *start_file;
 	bool          execute_cgi;
 	bool          execute_cgiset;
-	bool          show_index;
-	bool          show_index_set;
 #ifdef HAVE_XSLT
-	char          *index_xslt;
+	char          *show_index;
+	bool          show_index_set;
 #endif
 	bool          follow_symlinks;
 	bool          follow_symlinks_set;
@@ -172,16 +187,14 @@ typedef struct type_host {
 	bool          user_websites;
 	bool          execute_cgi;
 	int           time_for_cgi;
-	bool          show_index;
 #ifdef HAVE_XSLT
-	char          *index_xslt;
+	char          *show_index;
 #endif
 	bool          use_gz_file;
 	char          *login_message;
 	char          *passwordfile;
 	t_auth_method auth_method;
 	char          *groupfile;
-	char          *index_style;
 	t_charlist    required_binding;
 	t_denybotlist *deny_bot;
 	t_charlist    required_group;
@@ -206,7 +219,6 @@ typedef struct type_host {
 #ifdef HAVE_XSLT
 	bool          use_xslt;
 #endif
-	bool          prevent_cmdi;
 	bool          prevent_sqli;
 	bool          prevent_xss;
 	bool          prevent_csrf;
@@ -219,12 +231,20 @@ typedef struct type_host {
 	struct type_host *next;
 
 	bool          secure_url;
+
+#ifdef HAVE_MONITOR
+	t_monitor_host_stats *monitor_stats;
+	bool          monitor_requests;
+	bool          monitor_host;
+#endif
 } t_host;
 
 typedef struct type_config {
 	char          *mimetype_config;
 
-	char          *server_root;
+#ifdef HAVE_CHROOT
+    char          *server_root;
+#endif
 	uid_t         server_uid;
 	gid_t         server_gid;
 	t_groups      groups;
@@ -270,7 +290,6 @@ typedef struct type_config {
 	int           ban_on_max_per_ip;
 	int           ban_on_flooding;
 	int           ban_on_max_request_size;
-	int           ban_on_cmdi;
 	int           ban_on_sqli;
 	int           ban_on_timeout;
 	int           ban_on_wrong_password;
@@ -283,9 +302,9 @@ typedef struct type_config {
 	t_accesslist  *banlist_mask;
 	t_fcgi_server *fcgi_server;
 
+	char          *work_directory;
 	char          *upload_directory;
 	size_t        upload_directory_len;
-	bool          create_upload_dir;
 
 #ifdef HAVE_CACHE
 	off_t         cache_size;
@@ -296,6 +315,13 @@ typedef struct type_config {
 #ifdef HAVE_COMMAND
 	t_binding     *command_port;
 #endif
+
+#ifdef HAVE_MONITOR
+	bool          monitor_enabled;
+	char          *monitor_directory;
+	int           monitor_stats_interval;
+	t_monitor_srv_stats monitor_stats;
+#endif
 } t_config;
 
 t_config *default_config(void);
