@@ -105,6 +105,7 @@ void init_session(t_session *session) {
 	session->ssl_data = NULL;
 #endif
 
+	session->last_host = NULL;
 	session->request = NULL;
 	session->buffer_size = 0;
 	session->bytes_in_buffer = 0;
@@ -228,7 +229,7 @@ int get_target_extension(t_session *session) {
 	}
 
 	if ((session->extension = strrchr(last_slash, '.')) != NULL) {
-		(session->extension)++;
+		session->extension++;
 	} else {
 		session->extension = session->config->no_extension_as;
 	}
