@@ -22,8 +22,9 @@
 #include "libfs.h"
 #include "alternative.h"
 
-#define MAX_RULE_LOOP   20
-#define MAX_MATCH_LOOP  20
+#define REGEXEC_NMATCH 10
+#define MAX_SUB_DEPTH  10
+#define MAX_MATCH_LOOP 20
 
 t_url_toolkit *find_toolkit(char *toolkit_id, t_url_toolkit *url_toolkit) {
 	if (toolkit_id == NULL) {
@@ -69,8 +70,8 @@ bool toolkit_setting(char *key, char *value, t_url_toolkit *toolkit) {
 	}
 
 	if (strcmp(key, "toolkitid") == 0) {
-		if ((toolkit->toolkit_id = strdup(value)) != NULL) {
-			return true;
+		if ((toolkit->toolkit_id = strdup(value)) == NULL) {
+			return false;
 		}
 	} else {
 		if ((new_rule = (t_toolkit_rule*)malloc(sizeof(t_toolkit_rule))) == NULL) {
@@ -84,14 +85,20 @@ bool toolkit_setting(char *key, char *value, t_url_toolkit *toolkit) {
 			}
 			rule->next = new_rule;
 		}
-		new_rule->parameter = NULL;
-		new_rule->done = td_continue;
+
+		new_rule->condition = tc_none;
+		new_rule->operation = to_none;
+		new_rule->flow = tf_continue;
+		new_rule->conditional_flow = tf_continue;
 		new_rule->match_loop = 1;
+		new_rule->parameter = NULL;
+		new_rule->value = 0;
 		new_rule->next = NULL;
 
 		if (strcmp(key, "match") == 0) {
 			/* Match
 			 */
+			new_rule->condition = tc_match;
 			if (split_string(value, &value, &rest, ' ') == -1) {
 				return false;
 			} else if (regcomp(&(new_rule->pattern), value, REG_EXTENDED) != 0) {
@@ -100,17 +107,17 @@ bool toolkit_setting(char *key, char *value, t_url_toolkit *toolkit) {
 			split_string(rest, &value, &rest, ' ');
 			
 			if (strcasecmp(value, "ban") == 0) {
-				/* Match UseFastCGI
+				/* Match Ban
 				 */
-				new_rule->action = ta_match_ban;
+				new_rule->operation = to_ban;
 				if ((new_rule->value = str2int(rest)) == false) {
 					return false;
 				}
 			} else if (strcasecmp(value, "call") == 0) {
 				/* Match Call
 				 */
-				new_rule->action = ta_match_call;
-				if (strlen(rest) == 0) {
+				new_rule->operation = to_sub;
+				if (rest == NULL) {
 					return false;
 				} else if ((new_rule->parameter = strdup(rest)) == NULL) {
 					return false;
@@ -118,15 +125,16 @@ bool toolkit_setting(char *key, char *value, t_url_toolkit *toolkit) {
 			} else if (strcasecmp(value, "denyaccess") == 0) {
 				/* Match DenyAccess
 				 */
-				new_rule->action = ta_match_deny_access;
+				new_rule->operation = to_denyaccess;
+				new_rule->flow = tf_exit;
 			} else if (strcasecmp(value, "exit") == 0) {
 				/* Match Exit
 				 */
-				new_rule->action = ta_match_exit;
+				new_rule->flow = tf_exit;
 			} else if (strcasecmp(value, "expire") == 0) {
 				/* Match Expire
 				 */
-				new_rule->action = ta_match_expire;
+				new_rule->operation = to_expire;
 				if (split_string(rest, &value, &rest, ' ') == -1) {
 					return false;
 				}
@@ -155,20 +163,21 @@ bool toolkit_setting(char *key, char *value, t_url_toolkit *toolkit) {
 					return false;
 				}
 
-				if (rest == NULL) {
-					new_rule->done = td_continue;
-				} else if (strcasecmp(rest, "exit") == 0) {
-					new_rule->done = td_exit;
-				} else if (strcasecmp(rest, "return") == 0) {
-					new_rule->done = td_return;
-				} else {
-					return false;
+				if (rest != NULL) {
+					if (strcasecmp(rest, "exit") == 0) {
+						new_rule->flow = tf_exit;
+					} else if (strcasecmp(rest, "return") == 0) {
+						new_rule->flow = tf_return;
+					} else {
+						return false;
+					}
 				}
 			} else if (strcasecmp(value, "goto") == 0) {
 				/* Match Goto
 				 */
-				new_rule->action = ta_match_goto;
-				if (strlen(rest) == 0) {
+				new_rule->operation = to_sub;
+				new_rule->flow = tf_exit;
+				if (rest == NULL) {
 					return false;
 				} else if ((new_rule->parameter = strdup(rest)) == NULL) {
 					return false;
@@ -176,8 +185,9 @@ bool toolkit_setting(char *key, char *value, t_url_toolkit *toolkit) {
 			} else if (strcasecmp(value, "redirect") == 0) {
 				/* Match Redirect
 				 */
-				new_rule->action = ta_match_redirect;
-				if (strlen(rest) == 0) {
+				new_rule->operation = to_redirect;
+				new_rule->flow = tf_exit;
+				if (rest == NULL) {
 					return false;
 				} else if ((new_rule->parameter = strdup(rest)) == NULL) {
 					return false;
@@ -185,15 +195,15 @@ bool toolkit_setting(char *key, char *value, t_url_toolkit *toolkit) {
 			} else if (strcasecmp(value, "return") == 0) {
 				/* Match Return
 				 */
-				new_rule->action = ta_match_return;
+				new_rule->flow = tf_return;
 			} else if (strcasecmp(value, "rewrite") == 0) {
 				/* Match Rewrite
 				 */
-				new_rule->action = ta_match_rewrite;
-				new_rule->done = td_exit;
+				new_rule->operation = to_rewrite;
+				new_rule->flow = tf_exit;
 
 				split_string(rest, &value, &rest, ' ');
-				if (strlen(value) == 0) {
+				if (value == NULL) {
 					return false;
 				} else if ((new_rule->parameter = strdup(value)) == NULL) {
 					return false;
@@ -204,18 +214,19 @@ bool toolkit_setting(char *key, char *value, t_url_toolkit *toolkit) {
 					if ((loop = str2int(value)) > 0) {
 						if (loop > MAX_MATCH_LOOP) {
 							return false;
-						} else if ((value = rest) == NULL) {
-							return false;
 						}
 						new_rule->match_loop = loop;
+						if ((value = rest) == NULL) {
+							return true;
+						}
 					} else if (rest != NULL) {
 						return false;
 					}
 
 					if (strcasecmp(value, "continue") == 0) {
-						new_rule->done = td_continue;
+						new_rule->flow = tf_continue;
 					} else if (strcasecmp(value, "return") == 0) {
-						new_rule->done = td_return;
+						new_rule->flow = tf_return;
 					} else {
 						return false;
 					}
@@ -223,23 +234,29 @@ bool toolkit_setting(char *key, char *value, t_url_toolkit *toolkit) {
 			} else if (strcasecmp(value, "skip") == 0) {
 				/* Match Skip
 				 */
-				new_rule->action = ta_match_skip;
+				new_rule->operation = to_skip;
 				if ((new_rule->value = str2int(rest)) < 1) {
 					return false;
 				}
 			} else if (strcasecmp(value, "usefastcgi") == 0) {
 				/* Match UseFastCGI
 				 */
-				new_rule->action = ta_match_fastcgi;
-				new_rule->parameter = strdup(rest);
+				new_rule->operation = to_fastcgi;
+				new_rule->flow = tf_exit;
+				if (rest == NULL) {
+					return false;
+				} else if ((new_rule->parameter = strdup(rest)) == NULL) {
+					return false;
+				}
 			} else {
 				return false;
 			}
 		} else if (strcmp(key, "call") == 0) {
 			/* Call
 			 */
-			new_rule->action = ta_call;
-			if (strlen(value) == 0) {
+			new_rule->operation = to_sub;
+
+			if (value == NULL) {
 				return false;
 			} else if ((new_rule->parameter = strdup(value)) == NULL) {
 				return false;
@@ -247,17 +264,22 @@ bool toolkit_setting(char *key, char *value, t_url_toolkit *toolkit) {
 		} else if (strcmp(key, "skip") == 0) {
 			/* Skip
 			 */
-			new_rule->action = ta_skip;
-			if ((new_rule->value = str2int(value)) < 1) {
+			new_rule->operation = to_skip;
+
+			if (value == NULL) {
+				return false;
+			} else if ((new_rule->value = str2int(value)) < 1) {
 				return false;
 			}
 		} else if (strcmp(key, "requesturi") == 0) {
 			/* RequestURI
 			 */
-			new_rule->action = ta_requesturi;
+			new_rule->condition = tc_requesturi;
+
 			if (split_string(value, &value, &rest, ' ') == -1) {
 				return false;
 			}
+
 			if (strcasecmp(value, "exists") == 0) {
 				new_rule->value = IU_EXISTS;
 			} else if (strcasecmp(value, "isfile") == 0) {
@@ -267,26 +289,67 @@ bool toolkit_setting(char *key, char *value, t_url_toolkit *toolkit) {
 			} else {
 				return false;
 			}
+
 			if (strcasecmp(rest, "return") == 0) {
-				new_rule->done = td_return;
+				new_rule->conditional_flow = tf_return;
 			} else if (strcasecmp(rest, "exit") == 0) {
-				new_rule->done = td_exit;
+				new_rule->conditional_flow = tf_exit;
 			} else {
 				return false;
 			}
+#ifdef HAVE_SSL
 		} else if (strcmp(key, "usessl") == 0) {
 			/* UseSSL
 			 */
-		new_rule->action = ta_usessl;
+			new_rule->condition = tc_usessl;
+			split_string(value, &value, &rest, ' ');
 
+			if (strcasecmp(value, "call") == 0) {
+				/* UseSSL Call
+				 */
+				new_rule->operation = to_sub;
+				if (rest == NULL) {
+					return false;
+				} else if ((new_rule->parameter = strdup(rest)) == NULL) {
+					return false;
+				}
+			} else if (strcasecmp(value, "exit") == 0) {
+				/* UseSSL Exit
+				 */
+				new_rule->flow = tf_exit;
+			} else if (strcasecmp(value, "goto") == 0) {
+				/* UseSSL Goto
+				 */
+				new_rule->operation = to_sub;
+				new_rule->flow = tf_exit;
+				if (rest == NULL) {
+					return false;
+				} else if ((new_rule->parameter = strdup(rest)) == NULL) {
+					return false;
+				}
+			} else if (strcasecmp(value, "return") == 0) {
+				/* UseSSL Return
+				 */
+				new_rule->flow = tf_return;
+			} else if (strcasecmp(value, "skip") == 0) {
+				/* UseSSL Skip
+				 */
+				new_rule->operation = to_skip;
+				if ((new_rule->value = str2int(rest)) < 1) {
+					return false;
+				}
+			} else {
+				return false;
+			}
+#endif
 		} else {
+			/* Unknown condition
+			 */
 			return false;
 		}
-
-		return true;
 	}
 
-	return false;
+	return true;
 }
 
 bool toolkit_rules_oke(t_url_toolkit *url_toolkit) {
@@ -302,7 +365,7 @@ bool toolkit_rules_oke(t_url_toolkit *url_toolkit) {
 
 		rule = toolkit->toolkit_rule;
 		while (rule != NULL) {
-			if ((rule->action == ta_match_goto) || (rule->action == ta_match_call) || (rule->action == ta_call)) {
+			if (rule->operation == to_sub) {
 				if (rule->parameter == NULL) {
 					fprintf(stderr, "Missing parameter in toolkit rule '%s'.\n", toolkit->toolkit_id);
 					return false;
@@ -319,11 +382,10 @@ bool toolkit_rules_oke(t_url_toolkit *url_toolkit) {
 	return true;
 }
 
-static int do_rewrite(char *url, regex_t *regexp, char *rep, char **new_url, int loop) {
+static int do_rewrite(char *url, regex_t *regexp, regmatch_t *pmatch, char *rep, char **new_url, int loop) {
 	int ofs, len, len_rep, i, n;
-	regmatch_t pmatch[10];
 	char *repl, *c, *sub, *tmp;
-	bool replaced;
+	bool first_run = true, replaced;
 
 	if ((url == NULL) || (regexp == NULL) || (rep == NULL) || (new_url == NULL)) {
 		return -1;
@@ -331,9 +393,12 @@ static int do_rewrite(char *url, regex_t *regexp, char *rep, char **new_url, int
 
 	*new_url = NULL;
 	while (loop-- > 0) {
-		if (regexec(regexp, url, 10, pmatch, 0) == REG_NOMATCH) {
+		if (first_run) {
+			first_run = false;
+		} else if (regexec(regexp, url, REGEXEC_NMATCH, pmatch, 0) == REG_NOMATCH) {
 			break;
 		}
+
 		if ((ofs = pmatch[0].rm_so) == -1) {
 			return -1;
 		}
@@ -396,6 +461,7 @@ static int do_rewrite(char *url, regex_t *regexp, char *rep, char **new_url, int
 }
 
 void init_toolkit_options(t_toolkit_options *options, char *website_root, t_url_toolkit *toolkit, bool use_ssl) {
+	options->sub_depth = 0;
 	options->new_url = NULL;
 	options->website_root = website_root;
 	options->fastcgi_server = NULL;
@@ -408,10 +474,11 @@ void init_toolkit_options(t_toolkit_options *options, char *website_root, t_url_
 int use_toolkit(char *url, char *toolkit_id, t_toolkit_options *options) {
 	t_url_toolkit *toolkit;
 	t_toolkit_rule *rule;
-	bool replaced = false;
-	int loop = 0, result, skip = 0;
+	bool condition_oke, replaced = false;
+	int result, skip = 0;
 	t_fsbool is_dir;
 	char *file, *qmark;
+	regmatch_t pmatch[REGEXEC_NMATCH];
 
 	if (options == NULL) {
 		return UT_ERROR;
@@ -425,133 +492,34 @@ int use_toolkit(char *url, char *toolkit_id, t_toolkit_options *options) {
 
 	rule = toolkit->toolkit_rule;
 	while (rule != NULL) {
+		condition_oke = false;
+
+		/* Skip lines
+		 */
 		if (skip > 0) {
 			skip--;
-		} else switch (rule->action) {
-			case ta_match_rewrite:
-				if (do_rewrite(url, &(rule->pattern), rule->parameter, &(options->new_url), rule->match_loop) == -1) {
-					if (options->new_url != NULL) {
-						free(options->new_url);
-						options->new_url = NULL;
-					}
-					return UT_ERROR;
-				}
-				if (options->new_url != NULL) {
-					if (replaced) {
-						free(url);
-					}
-					url = options->new_url;
-					replaced = true;
+			rule = rule->next;
+			continue;
+		}
 
-					if (rule->done == td_return) {
-						return UT_RETURN;
-					} else if (rule->done == td_exit) {
-						return UT_EXIT;
-					}
-				} else if (replaced) {
-					options->new_url = url;
+		/* Condition
+		 */
+		switch (rule->condition) {
+			case tc_none:
+				/* None
+				 */
+				condition_oke = true;
+				break;
+			case tc_match:
+				/* Match
+				 */
+				if (regexec(&(rule->pattern), url, REGEXEC_NMATCH, pmatch, 0) == 0) {
+					condition_oke = true;
 				}
 				break;
-			case ta_match_redirect:
-				if (do_rewrite(url, &(rule->pattern), rule->parameter, &(options->new_url), rule->match_loop) == -1) {
-					if (options->new_url != NULL) {
-						free(options->new_url);
-						options->new_url = NULL;
-					}
-					return UT_ERROR;
-				}
-				if (options->new_url != NULL) {
-					if (replaced) {
-						free(url);
-					}
-					return UT_REDIRECT;
-				} else if (replaced) {
-					options->new_url = url;
-				}
-				break;
-			case ta_call:
-			case ta_match_call:
-			case ta_match_goto:
-				if (rule->action != ta_call) {
-					if (regexec(&(rule->pattern), url, 0, NULL, 0) == REG_NOMATCH) {
-						break;
-					}
-				}
-				if (++loop == MAX_RULE_LOOP) {
-					return UT_ERROR;
-				}
-
-				if ((result = use_toolkit(url, rule->parameter, options)) == UT_ERROR) {
-					if (options->new_url != NULL) {
-						free(options->new_url);
-						options->new_url = NULL;
-					}
-					return UT_ERROR;
-				}
-
-				if (options->new_url != NULL) {
-					if (replaced) {
-						free(url);
-					}
-					url = options->new_url;
-					replaced = true;
-				} else if (replaced) {
-					options->new_url = url;
-				}
-
-				if ((result == UT_DENY_ACCESS) || (result == UT_EXIT) || (result == UT_REDIRECT)) {
-					return result;
-				} else if (rule->action == ta_match_goto) {
-					return UT_RETURN;
-				}
-				break;
-			case ta_match_ban:
-				if (regexec(&(rule->pattern), url, 0, NULL, 0) == 0) {
-					options->ban = rule->value;
-					return UT_EXIT;
-				}
-				break;
-			case ta_match_deny_access:
-				if (regexec(&(rule->pattern), url, 0, NULL, 0) == 0) {
-					return UT_DENY_ACCESS;
-				}
-				break;
-			case ta_match_exit:
-				if (regexec(&(rule->pattern), url, 0, NULL, 0) == 0) {
-					return UT_EXIT;
-				}
-				break;
-			case ta_match_expire:
-				if (regexec(&(rule->pattern), url, 0, NULL, 0) == 0) {
-					options->expire = rule->value;
-
-					if (rule->done == td_return) {
-						return UT_RETURN;
-					} else if (rule->done == td_exit) {
-						return UT_EXIT;
-					}
-				}
-				break;
-			case ta_match_fastcgi:
-				if (regexec(&(rule->pattern), url, 0, NULL, 0) == 0) {
-					options->fastcgi_server = rule->parameter;
-					return UT_EXIT;
-				}
-				break;
-			case ta_match_return:
-				if (regexec(&(rule->pattern), url, 0, NULL, 0) == 0) {
-					return UT_RETURN;
-				}
-				break;
-			case ta_match_skip:
-				if (regexec(&(rule->pattern), url, 0, NULL, 0) == 0) {
-					skip = rule->value;
-				}
-				break;
-			case ta_skip:
-				skip = rule->value;
-				break;
-			case ta_requesturi:
+			case tc_requesturi:
+				/* Request URI
+				 */
 				if (valid_uri(url) == false) {
 					break;
 				}
@@ -569,28 +537,159 @@ int use_toolkit(char *url, char *toolkit_id, t_toolkit_options *options) {
 				switch (rule->value) {
 					case IU_EXISTS:
 						if ((is_dir == yes) || (is_dir == no)) {
-							return (rule->done == td_return ? UT_RETURN : UT_EXIT);
+							rule->flow = rule->conditional_flow;
+							condition_oke = true;
 						}
 						break;
 					case IU_ISFILE:
 						if (is_dir == no) {
-							return (rule->done == td_return ? UT_RETURN : UT_EXIT);
+							rule->flow = rule->conditional_flow;
+							condition_oke = true;
 						}
 						break;
 					case IU_ISDIR:
 						if (is_dir == yes) {
-							return (rule->done == td_return ? UT_RETURN : UT_EXIT);
+							rule->flow = rule->conditional_flow;
+							condition_oke = true;
 						}
 						break;
 				}
 				break;
-			case ta_usessl:
-				if (options->use_ssl == false) {
-					break;
+#ifdef HAVE_SSL
+			case tc_usessl:
+				/* Client connections uses SSL?
+				 */
+				condition_oke = options->use_ssl;
+				break;
+#endif
+		}
+
+		/* Condition not oke
+		 */
+		if (condition_oke == false) {
+			rule = rule->next;
+			continue;
+		}
+
+		/* Operation
+		 */
+		switch (rule->operation) {
+			case to_none:
+				/* None
+				 */
+				break;
+			case to_rewrite:
+				/* Rewrite
+				 */
+				if (do_rewrite(url, &(rule->pattern), pmatch, rule->parameter, &(options->new_url), rule->match_loop) == -1) {
+					if (options->new_url != NULL) {
+						free(options->new_url);
+						options->new_url = NULL;
+					}
+					return UT_ERROR;
 				}
-				switch (rule->value)
+				if (options->new_url != NULL) {
+					if (replaced) {
+						free(url);
+					}
+					url = options->new_url;
+					replaced = true;
+				} else if (replaced) {
+					options->new_url = url;
+				}
+				break;
+			case to_sub:
+				/* Subroutine
+				 */
+				if (++(options->sub_depth) > MAX_SUB_DEPTH) {
+					return UT_ERROR;
+				}
+
+				if ((result = use_toolkit(url, rule->parameter, options)) == UT_ERROR) {
+					if (options->new_url != NULL) {
+						free(options->new_url);
+						options->new_url = NULL;
+					}
+					return UT_ERROR;
+				}
+				options->sub_depth--;
+
+				if (options->new_url != NULL) {
+					if (replaced) {
+						free(url);
+					}
+					url = options->new_url;
+					replaced = true;
+				} else if (replaced) {
+					options->new_url = url;
+				}
+
+				if (result != UT_RETURN) {
+					return result;
+				}
+				break;
+			case to_expire:
+				/* Send Expire HTTP header
+				 */
+				options->expire = rule->value;
+				break;
+			case to_skip:
+				/* Skip
+				 */
+				skip = rule->value;
+				break;
+			case to_denyaccess:
+				/* Deny access
+				 */
+				return UT_DENY_ACCESS;
+			case to_redirect:
+				/* Redirect client
+				 */
+				if (do_rewrite(url, &(rule->pattern), pmatch, rule->parameter, &(options->new_url), rule->match_loop) == -1) {
+					if (options->new_url != NULL) {
+						free(options->new_url);
+						options->new_url = NULL;
+					}
+					return UT_ERROR;
+				}
+				if (options->new_url != NULL) {
+					if (replaced) {
+						free(url);
+					}
+					return UT_REDIRECT;
+				} else if (replaced) {
+					options->new_url = url;
+				}
+				break;
+			case to_fastcgi:
+				/* Use FastCGI server
+				 */
+				options->fastcgi_server = rule->parameter;
+				break;
+			case to_ban:
+				/* Ban client
+				 */
+				options->ban = rule->value;
 				break;
 		}
+
+		/* Flow
+		 */
+		switch (rule->flow) {
+			case tf_continue:
+				/* Continue
+				 */
+				break;
+			case tf_exit:
+				/* Exit
+				 */
+				return UT_EXIT;
+			case tf_return:
+				/* Return
+				 */
+				return UT_RETURN;
+		}
+
 		rule = rule->next;
 	}
 
